
#include <stdlib.h>
#include <allegro.h>
#include <time.h>

#ifdef ALLEGRO_PSP
	#define COLORDEPTH 32
	#define WIDTH 480
	#define HEIGHT 272
	#define DRIVER GFX_AUTODETECT
#else
	#define COLORDEPTH 32
	#define WIDTH 640
	#define HEIGHT 480
	#define DRIVER GFX_AUTODETECT
#endif

#define MAX_LIGHTS (WIDTH*HEIGHT/3800)

typedef struct Light
{
	RGB color;
	int x, y;
	int dx, dy;
	int tx, ty;
	int life;
	int size;
} Light;

Light *create_light()
{
	Light *newlight = malloc(sizeof(Light));
	if (rand()%2)
	{
		newlight->x = 0;
		newlight->y = rand()%SCREEN_H;
	}
	else
	{
		newlight->x = rand()%SCREEN_W;
		newlight->y = 0;
	}
	newlight->dx = rand()%30 - 15;
	if (newlight->dx == 0)
	{
		newlight->dx = 1;
	}
	newlight->dy = rand()%30 - 15;
	if (newlight->dy == 0)
	{
		newlight->dy = 1;
	}
	newlight->life = rand()%40 + 60;
	newlight->color.r = rand()%200 + 55;
	newlight->color.g = rand()%200 + 55;
	newlight->color.b = rand()%200 + 55;
	newlight->size = rand()%8 + 12;
	return newlight;
}

inline long signed int _abs(long signed int val)
{
	if (val < 0)
	{
		return 0 - val;
	}
	else
	{
		return val;
	}
}

inline double _fabs(double val)
{
	if (val < 0.0)
	{
		return 0.0 - val;
	}
	else
	{
		return val;
	}
}

inline long signed int _rabs(long signed int val)
{
	if (val > 0)
	{
		return 0 - val;
	}
	else
	{
		return val;
	}
}

inline double _rfabs(double val)
{
	if (val > 0.0)
	{
		return 0.0 - val;
	}
	else
	{
		return val;
	}
}

void render_lights(BITMAP *buffer, Light *lights[])
{
	int c = 0;
	signed int i, j;
	unsigned long int color;
	unsigned char r, g, b;
	int blinker;
	Light *p;

	while (c < MAX_LIGHTS)
	{
		p = lights[c++];
		for (i = 0-p->size/2-2; i < p->size/2+2; ++i)
		{
			for (j = 0-p->size/2-2; j < p->size/2+2; ++j)
			{
				color = getpixel(buffer,p->x-i,p->y-j);
				r = getr(color);
				g = getg(color);
				b = getb(color);
				blinker = rand()%2 + 1;
				r += p->color.r/(_rabs(i*20/p->size)*blinker-1)/(_rabs(j*20/p->size)*blinker-1);
				g += p->color.g/(_rabs(i*20/p->size)*blinker-1)/(_rabs(j*20/p->size)*blinker-1);
				b += p->color.b/(_rabs(i*20/p->size)*blinker-1)/(_rabs(j*20/p->size)*blinker-1);
				if (p->life < 15)
				{
					r /= 2;
					g /= 2;
					b /= 2;
				}
				if (p->life < 10)
				{
					r /= 2;
					g /= 2;
					b /= 2;
				}
				if (p->life < 5)
				{
					r /= 2;
					g /= 2;
					b /= 2;
				}
				if (p->life < 3)
				{
					r /= 2;
					g /= 2;
					b /= 2;
				}
				if (p->life < 2)
				{
					r /= 2;
					g /= 2;
					b /= 2;
				}
				color = makecol(r,g,b);
				putpixel(buffer,p->x-i,p->y-j,color);
			}
		}

		if (rand()%2)
		{
			--p->life;
		}
		if (p->life < 0)
		{
			if (rand()%2)
			{
				p->x = 0;
				p->y = rand()%SCREEN_H;
			}
			else
			{
				p->x = rand()%SCREEN_W;
				p->y = 0;
			}
			p->dx = rand()%30 - 15;
			if (p->dx == 0)
			{
				p->dx = 1;
			}
			p->dy = rand()%30 - 15;
			if (p->dy == 0)
			{
				p->dy = 1;
			}
			p->life = rand()%40 + 60;
			p->color.r = rand()%200 + 55;
			p->color.g = rand()%200 + 55;
			p->color.b = rand()%200 + 55;
			p->size = rand()%8 + 12;
		}
		p->x += p->dx/6;
		p->y += p->dy/6;

		if (p->x < 10 || p->x > SCREEN_W+10)
		{
			p->x = rand()%SCREEN_W;
		}
		if (p->y < 10 || p->y > SCREEN_H+10)
		{
			p->y = rand()%SCREEN_H;
		}
	}
}

int main(void)
{
	int i;
	Light *lights[MAX_LIGHTS];
	BITMAP *buffer;

	allegro_init();
	install_mouse();

	set_color_depth(COLORDEPTH);
	set_gfx_mode(DRIVER,WIDTH,HEIGHT,0,0);
	srand(time(NULL)+clock());

	for (i = 0; i < MAX_LIGHTS; ++i)
	{
		lights[i] = create_light();
	}

	buffer = create_bitmap(SCREEN_W,SCREEN_H);

	while (!(mouse_b & 2))
	{
		poll_mouse();
		render_lights(buffer,lights);
		blit(buffer,screen,0,0,0,0,SCREEN_W,SCREEN_H);
		clear_bitmap(buffer);
		rest(50);
	}

	allegro_exit();
	return 0;
}
END_OF_MAIN()
